RSpec.describe FaradayMiddleware::Avro::EncodeAvro do
  subject { described_class.new application }

  let(:application) { double 'Application' }
  let(:environment) do
    {
      request_headers: request_headers,
      body: body,
    }
  end
  let(:request_headers) do
    {
      'Content-Type' => content_type,
      'Avro-Schema' => avro_schema,
    }
  end
  let(:content_type) { 'avro/binary' }
  let(:avro_schema) { 'person' }
  let(:body) { { name: 'Buck' } }
  let(:response) { double 'Response', on_complete: nil }
  let(:parsed_body) { double 'Binary', class: :binary }

  describe '#call' do
    before do
      allow_any_instance_of(AvroTurf).to receive(:encode)
                                     .and_return(parsed_body)
      expect(application).to receive(:call)
                         .with(environment)
                         .and_return(response)
    end

    it 'changes the body type' do
      subject.call(environment)

      expect(environment[:body].class).to be :binary
    end

    context 'when request content is not Avro' do
      let(:content_type) { 'text/json' }

      it 'doest not change the body type' do
        subject.call(environment)

        expect(environment[:body].class).to be body.class
      end
    end

    context 'when request has no body' do
      let(:body) { nil }

      it 'doest not change the body type' do
        subject.call(environment)

        expect(environment[:body].class).to be body.class
      end
    end

    context 'when request body responds to #to_str' do
      let(:body) { { name: 'Buck '}.to_json }

      it 'changes the body type' do
        subject.call(environment)

        expect(environment[:body].class).to be parsed_body.class
      end
    end
  end
end
