RSpec.describe FaradayMiddleware::Avro::ParseAvro do
  subject { described_class.new application }

  let(:application) { double 'Application' }
  let(:env) { { body: body, response_headers: response_headers } }
  let(:body) { double 'Binary', respond_to?: false, empty?: empty_body }
  let(:response_headers) do
    { 'Content-Type' => content_type, 'Avro-Schema' => schema_name }
  end
  let(:content_type) { 'avro/binary' }
  let(:schema_name) { 'person' }
  let(:empty_body) { false }

  before do
    allow(subject).to receive(:preserve_raw?).and_return false
  end

  describe '#process_response' do
    let(:parsed_body) { { person: { name: 'Buck' } } }

    before do
      expect(subject).to receive(:parse).with(body, schema_name)
                                        .and_return(parsed_body)
    end

    it 'rewrites body with parsed params' do
      subject.process_response(env)

      expect(env[:body]).to eq parsed_body
    end
  end

  describe '#parse' do
    it 'decodes response body with AvroTurf' do
      expect_any_instance_of(AvroTurf).to receive(:decode)
                                      .with(body, schema_name: schema_name)

      subject.parse(body, schema_name)
    end

    context 'when body is empty' do
      let(:empty_body) { true }

      it 'does not decode response body' do
        expect_any_instance_of(AvroTurf).to_not receive(:decode)

        subject.parse(body, schema_name)
      end
    end
  end
end
